package Evo;
use Evo::Base -strict;
use Evo::Manager;
use Evo::MDN;

use Exporter 'import';

my @MSN
  = qw(mdn_subscribe mdn_broadcast mdn_unsubscribe mdn_unsubscribe_from_all);
my @DSL  = qw(dsl_stash dsl_check_stash dsl_call dsl_depth);
my @FUNC = qw(asap);
our @EXPORT_OK = (@DSL, @FUNC, @MSN);
our %EXPORT_TAGS
  = (all => \@EXPORT_OK, dsl => \@DSL, mdn => \@MSN, func => \@FUNC);

our $VERSION = '0.0153';    # VERSION

sub asap : prototype(&@) { Evo::Manager->singleton->asap(@_) }

sub dsl_depth : prototype()         { Evo::Manager->singleton->dsl_depth; }
sub dsl_call : prototype($;@)       { Evo::Manager->singleton->dsl_call(@_); }
sub dsl_stash : prototype(;$$)      { Evo::Manager->singleton->dsl_stash(@_); }
sub dsl_check_stash : prototype(;$) { Evo::Manager->singleton->dsl_stash(@_); }

sub _mdn { state $mdn = Evo::MDN->new }
sub mdn_broadcast : prototype($$)           { _mdn->broadcast(@_); }
sub mdn_subscribe : prototype($$)           { _mdn->subscribe(@_); }
sub mdn_unsubscribe : prototype($$)         { _mdn->unsubscribe(@_); }
sub mdn_unsubscribe_from_all : prototype($) { _mdn->unsubscribe_from_all(@_); }

1;

# ABSTRACT: Perl Evo design pattern

__END__

=pod

=encoding UTF-8

=head1 NAME

Evo - Perl Evo design pattern

=head1 VERSION

version 0.0153

=head1 SYNOPSIS

  use Evo::Base -strict;
  use Evo ':all';


  # next tick
  our $state = 'non blocking';

  # always nb
  asap {
    say $state;
    local $state = 'blocking?';
    asap { say $state };
  };


  # dsl
  # prints 1;2;foo
  my @args = (1, 2);
  my $print = sub(@args) { local $, = ';'; say @args, dsl_stash 'foo' };

  dsl_call({foo => 'foo'}, @args, $print);

  require Evo::Manager;
  my $mngr = Evo::Manager->new;

  # currying
  my $debug = sub($evo, $next) {
    sub(@args) {
      say 'Got args: ', join ';', @args;
      $next->(@args);
      }
  };

  my $safe = sub($evo, $next) {
    sub(@args) {
      local $@;
      eval { $next->(@args) };
      warn "catched: $@" if $@;
      }
  };

  my $die = sub {die};
  my $curry = $mngr->curry_wrappers($debug, $safe);
  $curry->(sub { say "going to die"; die })->(1, 2);

  my $way = $mngr->build_way;
  my $train = $mngr->build_train(way => $way);
  $way->curry($curry)->add(sub { say 'step1'; })->add(sub { die "step 2" })
    ->add(sub { say "step 3"; });

  $train->start;

=head1 DESCRIPTION

  Tiny framework for organization non-trivial code into a simple logic.
  It includes benefits from functional and OO styles.

  Write fast, reusable and testable code.
  Familiar with OO? - Use it. Familiar with higher-order functions? - Use it.
  Familiar with non-blocking style? - Use it. Want DSL? - Use it.

  Do it the right way.

=head1 DESCRIPTION

  Attention. Don't use it right now. Wait for the stable release

=head1 METHODS

=head2 mdn_broadcast, mdn_subscribe, mdn_unsubscribe, mdn_unsubscribe_from_all

Calls corresponding L<Evo::MDN> methods passing a singleton as the first
argument

=head2 asap, dsl_depth, dsl_stash dsl_check_stash dsl_call dsl_depth

Calls corresponding L<Evo::Manager> methods passing a singleton as the first
argument

=head1 AUTHOR

alexbyk.com

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by alexbyk.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
